<?php

namespace Quivers\Integration\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;

class CurrencyMappingObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;

    protected $curlClient;
    protected $scopeConfig;

    /**
     * @param Logger $logger
     */
    
    const XML_PRODUCTGROUP_ID = 'quivers/parameters/ProductGroupId';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EURO_MARKETPLACE_ID = 'quivers_tax/marketplaces/euroMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';
    const XML_QUIVERS_UUID = 'quivers/parameters/quivers_uuid';


    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';

    public function __construct(

        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ){
    	$this->curlClient = $curl;
    	$this->scopeConfig = $scopeConfig;
    }

    public function execute (EventObserver $observer) {


        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope);
        $euroMarketPlaceId = $this->scopeConfig->getValue(self::XML_EURO_MARKETPLACE_ID, $storeScope);
        $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope);
        $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope);
        $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope);
        $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope);
        $productGroupId = $this->scopeConfig->getValue(self::XML_PRODUCTGROUP_ID, $storeScope);
        $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        
        if($quiversSandbox == 0) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";

        $product_group_id_array = explode(", ", $productGroupId);
        $product_group_id_list = [];
        foreach ($product_group_id_array as $productGroupId) {
            array_push($product_group_id_list, $productGroupId);
        }

        $allMarketPlaces = [];

        $marketPlaceIds = [
            "USD" => $usdMarketPlaceId,
            "CAD" => $cadMarketPlaceId,
            "EURO" => $euroMarketPlaceId,
            "AUD" => $audMarketPlaceId,
            "JPY" => $jpyMarketPlaceId,
            "GBP" => $gbpMarketPlaceId
        ];
        foreach($marketPlaceIds as $currency=>$mPlaceId) {
            if($mPlaceId) {
                $allMarketPlaces[] = [
                    'marketplace_id' => $mPlaceId,
                    'currency' => $currency,
                    'claiming_group_ids' => $product_group_id_list         
                ];
            }
        }

        if(count($allMarketPlaces)>0) {
            $quiversUID = $this->scopeConfig->getValue("quivers/parameters/quivers_uuid");

            $payload = [ "marketplaces" => $allMarketPlaces ];

	        $this->curlClient->addHeader("Content-Type", "application/json");
            $this->curlClient->addHeader("uuid", $quiversUID);
            $this->curlClient->post($profileUpdateUrl, json_encode($payload));
            $response = $this->curlClient->getBody();
            if($this->curlClient->getStatus() >= 400) {
                $errorMessage = 'An error occurred while updating the profile. ERROR: '.$response;
                throw new CouldNotSaveException(
                    __($errorMessage));
            }
        }
    }

} 