define([
        'jquery',
        'Magento_Payment/js/view/payment/cc-form',
        'Stripe',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/checkout-data'
    ],
    function ($, Component, stripe, quote, checkoutdata) {
        'use strict';
        var config=window.checkoutConfig.payment.quivers_integration;

        return Component.extend({
            defaults: {
                template: 'Quivers_Integration/payment/stripe-form',
                publicKey: config ? config.publicKey : '',
                title: config ? config.title: 'Pay By Card',
                storedCards: config ? config.stripeCards: [],
                selectedCard: config ? config.selectedCard : [],
                customerId: config ? config.customerId : [] ,
            },

            initialize: function () {
                var self = this;

                this._super();

                // Initialize Stripe object thinger
                this.stripe = Stripe(this.publicKey);
                this.elements = this.stripe.elements();
            },

            onCardElementRender: function()
            {
                var style = {
                base: {
                    // Add your base input styles here. For example:
                    fontSize: '16px',
                    color: "#32325d",
                  }
                };

                // Create an instance of the card Element.
                this.card = this.elements.create('card', {style: style, hidePostalCode: true});

                // Add an instance of the card Element into the `card-element` <div>.
                this.card.mount('#card-element');
                this.card.addEventListener('change', function(event) {
                    var displayError = document.getElementById('card-errors');
                    if (event.error) {
                        displayError.textContent = event.error.message;
                    } else {
                        displayError.textContent = '';
                    }
                });
            },

            context: function() {
                return this;
            },

            getCode: function() {
                return 'quivers_integration';
            },

            isActive: function() {
                return true;
            },
            isPlaceOrderActionAllowed: function() {
                return true;
            },

            handleStripeToken: function (result) {
                if (result.error) {
                    this.setToken(null);
                    var errorElement = document.getElementById('card-errors');
                    errorElement.textContent = result.error.message;
                } else {
                    this.setToken(result.source.id);
                    this.placeOrder()
                }
            },

            getToken: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-token');
                return element.val();
            },

            setToken: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-token');
                element.val(value);
            },

            placeOrder: function() {
                var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
                
                if(this.getToken()) {
                    this._super();
                }else if(paymentSourceId){
                    this.setToken(paymentSourceId);
                    this.placeOrder();
                }else {
                    var address = checkoutdata.getNewCustomerBillingAddress();
                    if (address == null) {
                        address = quote.shippingAddress();
                    }
                    var postcode = (address.postcode);
                    var payment_method_code = this.getCode();
                    this.stripe.createSource(this.card, {owner: {address: { postal_code: postcode }}}).then(
                        this.handleStripeToken.bind(this)
                    );
                }
                return false;
            },

            onPlaceOrderClick: function() {
                var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
                if(paymentSourceId){
                    this.setToken(paymentSourceId);
                    this.placeOrder();
                }
                var address = checkoutdata.getNewCustomerBillingAddress();
                if (address == null) {
                    address = quote.shippingAddress();
                }
                var postcode = (address.postcode);
                var payment_method_code = this.getCode();
                this.stripe.createSource(this.card, {owner: {address: { postal_code: postcode }}}).then(
                    this.handleStripeToken.bind(this)
                );
            },

            getTitle: function() {
                return this.title;
            },

            getData: function()
            {
                var data = {
                    'method': this.getCode(),
                };
                data.additional_data = {
                        'cc_stripejs_token': this.getToken(),
                        'cc_save_card': jQuery('#save_my_card_checkbox input').attr('checked'),
                    };

                return data;
            },

            getStoredCards: function() {
                return this.storedCards;
            },

            getSelectedCard: function() {
                return this.selectedCard.label;
            },

            getCardSelectVisibility: function() {
                return this.storedCards.length
            },

            getSelectedCardFromList: function() {
                var selected = jQuery('#'+this.getCode()+'-card-id').find(":selected");
                return selected;
            },

            getFormElementVisibility: function() {
                var selected = this.getSelectedCardFromList();
                if(!this.getCardSelectVisibility() || !selected.attr('value')) return 1;
                else return 0;
            },

            getSaveCardElementVisibility: function() {
                if(!this.customerId) return 0;
                return this.getFormElementVisibility();
            },

            selectCardAfterRender: function() {
                //handle dropdown changes in card element
                var self = this;
                var cardElement = jQuery('#'+this.getCode()+'-card-id');
                cardElement.on('change', function() {
                    if(cardElement.children(':selected').attr('value')) {
                        jQuery('#'+self.getCode()+'-form').css('display', 'none');
                        jQuery('#save_my_card_checkbox').css('display', 'none');
                    }
                    else {
                        jQuery('#'+self.getCode()+'-form').css('display', 'block');
                        jQuery('#save_my_card_checkbox').css('display', 'block');                        
                    }
                });
            },
        });
    }
);
